import os

from genome import load


def test_genome(snp):
    test_dir = os.path.dirname(os.path.realpath(__file__))
    genome = load(os.path.join(str(test_dir), 'tt.txt'))
    assert len(genome) == 10
    assert repr(genome) == '<Genome: SNPs=10, name=\'tt.txt\'>'
    assert 'rs75333668' in genome
    assert genome['rs75333668'] == snp
    keys = genome.keys()
    values = genome.values()
    for key, val in zip(keys, values):
        assert genome[key] == val
    for key, val in genome.items():
        assert genome[key] == val
    assert [i for i in genome]
    assert len(genome.chromosome(1)) == 10
    assert len(genome.chromosome(2)) == 0


def test_snp(snp):
    assert repr(snp) == '<SNP: chromosome=\'1\' position=762320 genotype=<Genotype: \'CC\'>>'
    assert snp.chromosome == "1"
    assert snp.position == 762320


def test_genotype(genotype):
    assert genotype == "CC"
    assert repr(genotype) == '<Genotype: \'CC\'>'
    assert str(genotype) == 'CC'



# coding=utf-8
"""Genome"""

import os


class Genome:
    """
    Genome

    Parameters
    ----------
    name : str
        name of input file
    """
    def __init__(self, name):
        self.name = name
        self.dict = dict()

    def __setitem__(self, key, item):
        self.dict[key] = item

    def __getitem__(self, key):
        return self.dict[key]

    def __repr__(self):
        return '<Genome: SNPs={:d}, name={!r}>'.format(self.__len__(),
                                                       os.path.os.path.basename(self.name))

    def __len__(self):
        return len(self.dict)

    def __contains__(self, item):
        return item in self.dict

    def __iter__(self):
        return iter(self.dict)

    def keys(self):
        """

        Returns
        -------
        keys : dict_keys
            get keys (rsid)
        """
        return self.dict.keys()

    def values(self):
        """

        Returns
        -------
        values : dict_values
            get values (snp)
        """
        return self.dict.values()

    def items(self):
        """

        Returns
        -------
        items : dict_items
            get both key and values (rsid, snp)
        """
        return self.dict.items()

    def chromosome(self, key):
        """

        Parameters
        ----------
        key : str or int
            find all SNPs whose chromosome is the value of the key
        Returns
        -------
        SNPs : List
            list of SNPs
        """
        def search(snp, key):
            return snp if snp.chromosome == key else None
        return list(filter(lambda snp: search(snp, str(key)), self.dict.values()))
